# -*- coding: utf-8 -*-

from __future__ import print_function

""" Module for running planner portfolios.

Memory limits: We apply the same memory limit that is given to the
plan script to each planner call. Note that this setup does not work if
the sum of the memory usage of the Python process and the planner calls
is limited. In this case the Python process might get killed although
we would like to kill only the single planner call and continue with
the remaining configurations. If we ever want to support this scenario
we will have to reduce the memory limit of the planner calls by the
amount of memory that the Python process needs. On maia for example
this amounts to 128MB of reserved virtual memory. We can make Python
reserve less space by lowering the soft limit for virtual memory before
the process is started.
"""

__all__ = ["run"]

import os
import subprocess

from . import call
from . import limits
from . import returncodes
from . import util


DEFAULT_TIMEOUT = 1800


def adapt_heuristic_cost_type(arg, cost_type):
    if cost_type == "normal":
        transform = "no_transform()"
    else:
        transform = "adapt_costs({})".format(cost_type)
    return arg.replace("H_COST_TRANSFORM", transform)


def adapt_args(args, search_cost_type, heuristic_cost_type, plan_manager):
    g_bound = plan_manager.get_next_portfolio_cost_bound()
    plan_counter = plan_manager.get_plan_counter()
    print("g bound: %s" % g_bound)
    print("next plan number: %d" % (plan_counter + 1))

    for index, arg in enumerate(args):
        if arg == "--evaluator" or arg == "--heuristic":
            heuristic = args[index + 1]
            heuristic = adapt_heuristic_cost_type(heuristic, heuristic_cost_type)
            args[index + 1] = heuristic
        elif arg == "--search":
            search = args[index + 1]
            if "bound=BOUND" not in search:
                returncodes.exit_with_driver_critical_error(
                    "Satisficing portfolios need the string "
                    "\"bound=BOUND\" in each search configuration. "
                    "See the FDSS portfolios for examples.")
            for name, value in [
                    ("BOUND", g_bound),
                    ("S_COST_TYPE", search_cost_type)]:
                search = search.replace(name, str(value))
            search = adapt_heuristic_cost_type(search, heuristic_cost_type)
            args[index + 1] = search
            break


def run_search(executable, args, sas_file, plan_manager, time, memory):
    complete_args = [executable] + args + [
        "--internal-plan-file", plan_manager.get_plan_prefix()]
    print("args: %s" % complete_args)

    try:
        exitcode = call.check_call(
            "search", complete_args, stdin=sas_file,
            time_limit=time, memory_limit=memory)
    except subprocess.CalledProcessError as err:
        exitcode = err.returncode
    print("exitcode: %d" % exitcode)
    print()
    return exitcode


def compute_run_time(timeout, configs, pos):
    remaining_time = timeout - util.get_elapsed_time()
    print("remaining time: {}".format(remaining_time))
    relative_time = configs[pos][0]
    remaining_relative_time = sum(config[0] for config in configs[pos:])
    print("config {}: relative time {}, remaining {}".format(
          pos, relative_time, remaining_relative_time))
    # For the last config we have relative_time == remaining_relative_time, so
    # we use all of the remaining time at the end.
    return remaining_time * relative_time / remaining_relative_time


def run_sat_config(configs, pos, search_cost_type, heuristic_cost_type,
                   executable, sas_file, plan_manager, timeout, memory):
    run_time = compute_run_time(timeout, configs, pos)
    if run_time <= 0:
        return None
    _, args_template = configs[pos]
    args = list(args_template)
    adapt_args(args, search_cost_type, heuristic_cost_type, plan_manager)
    if not plan_manager.abort_portfolio_after_first_plan():
        args.extend([
            "--internal-previous-portfolio-plans",
            str(plan_manager.get_plan_counter())])
    result = run_search(executable, args, sas_file, plan_manager, run_time, memory)
    plan_manager.process_new_plans()
    return result


def run_sat(configs, executable, sas_file, plan_manager, final_config,
            final_config_builder, timeout, memory):
    # If the configuration contains S_COST_TYPE or H_COST_TRANSFORM and the task
    # has non-unit costs, we start by treating all costs as one. When we find
    # a solution, we rerun the successful config with real costs.
    heuristic_cost_type = "one"
    search_cost_type = "one"
    changed_cost_types = False
    while configs:
        configs_next_round = []
        for pos, (relative_time, args) in enumerate(configs):
            exitcode = run_sat_config(
                configs, pos, search_cost_type, heuristic_cost_type,
                executable, sas_file, plan_manager, timeout, memory)
            if exitcode is None:
                return

            yield exitcode
            if exitcode == returncodes.SEARCH_UNSOLVABLE:
                return

            if exitcode == returncodes.SUCCESS:
                if plan_manager.abort_portfolio_after_first_plan():
                    return
                configs_next_round.append((relative_time, args))
                if (not changed_cost_types and can_change_cost_type(args) and
                    plan_manager.get_problem_type() == "general cost"):
                    print("Switch to real costs and repeat last run.")
                    changed_cost_types = True
                    search_cost_type = "normal"
                    heuristic_cost_type = "plusone"
                    exitcode = run_sat_config(
                        configs, pos, search_cost_type, heuristic_cost_type,
                        executable, sas_file, plan_manager, timeout, memory)
                    if exitcode is None:
                        return

                    yield exitcode
                    if exitcode == returncodes.SEARCH_UNSOLVABLE:
                        return
                if final_config_builder:
                    print("Build final config.")
                    final_config = final_config_builder(args)
                    break

        if final_config:
            break

        # Only run the successful configs in the next round.
        configs = configs_next_round

    if final_config:
        print("Abort portfolio and run final config.")
        exitcode = run_sat_config(
            [(1, final_config)], 0, search_cost_type,
            heuristic_cost_type, executable, sas_file, plan_manager,
            timeout, memory)
        if exitcode is not None:
            yield exitcode


def run_opt(configs, executable, sas_file, plan_manager, timeout, memory):
    for pos, (relative_time, args) in enumerate(configs):
        run_time = compute_run_time(timeout, configs, pos)
        exitcode = run_search(executable, args, sas_file, plan_manager,
                              run_time, memory)
        yield exitcode

        if exitcode in [returncodes.SUCCESS, returncodes.SEARCH_UNSOLVABLE]:
            break


def can_change_cost_type(args):
    return any("S_COST_TYPE" in part or "H_COST_TRANSFORM" in part for part in args)


def get_portfolio_attributes(portfolio):
    attributes = {}
    with open(portfolio) as portfolio_file:
        content = portfolio_file.read()
        try:
            exec(content, attributes)
        except Exception:
            returncodes.exit_with_driver_critical_error(
                "The portfolio %s could not be loaded. Maybe it still "
                "uses the old portfolio syntax? See the FDSS portfolios "
                "for examples using the new syntax." % portfolio)
    if "CONFIGS" not in attributes:
        returncodes.exit_with_driver_critical_error("portfolios must define CONFIGS")
    if "OPTIMAL" not in attributes:
        returncodes.exit_with_driver_critical_error("portfolios must define OPTIMAL")
    return attributes


def run(portfolio, executable, sas_file, plan_manager, time, memory):
    """
    Run the configs in the given portfolio file.

    The portfolio is allowed to run for at most *time* seconds and may
    use a maximum of *memory* bytes.
    """
    attributes = get_portfolio_attributes(portfolio)
    configs = attributes["CONFIGS"]
    optimal = attributes["OPTIMAL"]
    final_config = attributes.get("FINAL_CONFIG")
    final_config_builder = attributes.get("FINAL_CONFIG_BUILDER")
    if "TIMEOUT" in attributes:
        returncodes.exit_with_driver_input_error(
            "The TIMEOUT attribute in portfolios has been removed. "
            "Please pass a time limit to fast-downward.py.")

    if time is None:
        if os.name == "nt":
            returncodes.exit_with_driver_unsupported_error(limits.RESOURCE_MODULE_MISSING_MSG)
        else:
            returncodes.exit_with_driver_input_error(
                "Portfolios need a time limit. Please pass --search-time-limit "
                "or --overall-time-limit to fast-downward.py.")

    timeout = util.get_elapsed_time() + time

    if optimal:
        exitcodes = run_opt(
            configs, executable, sas_file, plan_manager, timeout, memory)
    else:
        exitcodes = run_sat(
            configs, executable, sas_file, plan_manager, final_config,
            final_config_builder, timeout, memory)
    return returncodes.generate_portfolio_exitcode(exitcodes)



#! /usr/bin/env python

from lab.parser import Parser

parser = Parser()

LEGACY_TO_NEW_EXIT_CODES = {
    'critical-error': 'search-critical-error',
    'input-error': 'search-input-error',
    'unsupported-feature-requested': 'search-unsupported',
    'unsolvable': 'search-unsolvable',
    'incomplete-search-found-no-plan': 'search-unsolvable-incomplete',
    'out-of-memory': 'search-out-of-memory',
    'timeout': 'search-out-of-time',
    'timeout-and-out-of-memory': 'search-out-of-memory-and-time',
}

def convert_legacy_to_new_exit_codes(content, props):
    error = props['error']
    if error in LEGACY_TO_NEW_EXIT_CODES:
        props['error'] = LEGACY_TO_NEW_EXIT_CODES[error]

parser.add_function(convert_legacy_to_new_exit_codes)

parser.parse()



#! /usr/bin/env python

from lab.parser import Parser

def add_absolute_and_relative(parser, attribute, pattern):
    parser.add_pattern(attribute, pattern + ' (\d+) .+', required=False, type=int)
    parser.add_pattern(attribute + '_rel', pattern + ' \d+ \((.+)\)', required=False, type=float)


parser = Parser()

parser.add_pattern('sg_construction_time', 'SG construction time: (.+)s', required=False, type=float)
parser.add_pattern('sg_peak_mem_diff', 'SG construction peak memory difference: (\d+)', required=False, type=int)
parser.add_pattern('sg_size_estimate_total', 'SG size estimates: total: (\d+)', required=False, type=int)
add_absolute_and_relative(parser, 'sg_size_estimate_overhead', 'SG size estimates: object overhead:')
add_absolute_and_relative(parser, 'sg_size_estimate_operators', 'SG size estimates: operators:')
add_absolute_and_relative(parser, 'sg_size_estimate_switch_var', 'SG size estimates: switch var:')
add_absolute_and_relative(parser, 'sg_size_estimate_value_generator', 'SG size estimates: generator for value:')
add_absolute_and_relative(parser, 'sg_size_estimate_default_generator', 'SG size estimates: default generator:')
add_absolute_and_relative(parser, 'sg_size_estimate_next_generator', 'SG size estimates: next generator:')

add_absolute_and_relative(parser, 'sg_counts_immediates', 'SG object counts: immediates:')
add_absolute_and_relative(parser, 'sg_counts_forks', 'SG object counts: forks:')
add_absolute_and_relative(parser, 'sg_counts_switches', 'SG object counts: switches:')
add_absolute_and_relative(parser, 'sg_counts_leaves', 'SG object counts: leaves:')
add_absolute_and_relative(parser, 'sg_counts_empty', 'SG object counts: empty:')

add_absolute_and_relative(parser, 'sg_counts_switch_empty', 'SG switch statistics: immediate ops empty:')
add_absolute_and_relative(parser, 'sg_counts_switch_single', 'SG switch statistics: single immediate op:')
add_absolute_and_relative(parser, 'sg_counts_switch_more', 'SG switch statistics: more immediate ops:')

add_absolute_and_relative(parser, 'sg_counts_leaf_empty', 'SG leaf statistics: applicable ops empty:')
add_absolute_and_relative(parser, 'sg_counts_leaf_single', 'SG leaf statistics: single applicable op:')
add_absolute_and_relative(parser, 'sg_counts_leaf_more', 'SG leaf statistics: more applicable ops:')

add_absolute_and_relative(parser, 'sg_counts_switch_vector_single', 'SG switch statistics: vector single:')
add_absolute_and_relative(parser, 'sg_counts_switch_vector_small', 'SG switch statistics: vector small:')
add_absolute_and_relative(parser, 'sg_counts_switch_vector_large', 'SG switch statistics: vector large:')
add_absolute_and_relative(parser, 'sg_counts_switch_vector_full', 'SG switch statistics: vector full:')

parser.parse()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import downward.suites

import common_setup

CONFIGS = {
    "ehc_ff": [
        "--search", "ehc(ff())"],
    "ehc_add_pref": [
        "--heuristic", "hadd=add()", "--search", "ehc(hadd, preferred=[hadd])"],
    #"ehc_add_ff_pref": [
    #    "--search", "ehc(add(), preferred=[ff()],preferred_usage=RANK_PREFERRED_FIRST)"],
}

SUITE = downward.suites.suite_satisficing_with_ipc11()

exp = common_setup.IssueExperiment(
    search_revisions=["issue77-v6-base", "issue77-v6"],
    configs=CONFIGS,
    suite=SUITE
    )
exp.add_absolute_report_step()
exp.add_comparison_table_step()
# exp.add_scatter_plot_step()

exp()



from downward.reports import PlanningReport

class CSVReport(PlanningReport):
    def get_text(self):
        sep = " "
        lines = [sep.join(self.attributes)]
        for runs in self.problem_runs.values():
            for run in runs:
                lines.append(sep.join([str(run.get(attribute, "nan"))
                                       for attribute in self.attributes]))
        return "\n".join(lines)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import itertools
import os
import subprocess

from lab.environments import LocalEnvironment, BaselSlurmEnvironment

from downward.reports.compare import ComparativeReport

import common_setup
from common_setup import IssueConfig, IssueExperiment
from relativescatter import RelativeScatterPlotReport

EXPNAME = common_setup.get_experiment_name()
DIR = os.path.dirname(os.path.abspath(__file__))
BENCHMARKS_DIR = os.environ["DOWNWARD_BENCHMARKS"]
REVISIONS = ["issue213-v8"]
BUILDS = ["release32", "release64"]
CONFIGS = [
    IssueConfig(
        "lama-" + build,
        [],
        build_options=[build],
        driver_options=["--build", build, "--alias", "lama", "--overall-time-limit", "5m"])
    for rev in REVISIONS
    for build in BUILDS
]
SUITE = common_setup.DEFAULT_SATISFICING_SUITE
ENVIRONMENT = BaselSlurmEnvironment(
    partition="infai_1",
    email="jendrik.seipp@unibas.ch",
    export=["PATH", "DOWNWARD_BENCHMARKS"])

if common_setup.is_test_run():
    SUITE = IssueExperiment.DEFAULT_TEST_SUITE
    ENVIRONMENT = LocalEnvironment(processes=1)

exp = IssueExperiment(
    revisions=REVISIONS,
    configs=CONFIGS,
    environment=ENVIRONMENT,
)
exp.add_suite(BENCHMARKS_DIR, SUITE)

exp.add_parser(exp.EXITCODE_PARSER)
exp.add_parser(exp.ANYTIME_SEARCH_PARSER)
exp.add_parser(exp.PLANNER_PARSER)

exp.add_step("build", exp.build)
exp.add_step("start", exp.start_runs)
exp.add_fetcher(name="fetch")

#exp.add_absolute_report_step()
#exp.add_comparison_table_step()

attributes = IssueExperiment.PORTFOLIO_ATTRIBUTES

# Compare builds.
for build1, build2 in itertools.combinations(BUILDS, 2):
    for rev in REVISIONS:
        algorithm_pairs = [
            ("{rev}-{config_nick}-{build1}".format(**locals()),
             "{rev}-{config_nick}-{build2}".format(**locals()),
             "Diff ({config_nick}-{rev})".format(**locals()))
            for config_nick in ["lama"]]
        outfile = os.path.join(exp.eval_dir, "{EXPNAME}-{build1}-vs-{build2}.html".format(**locals()))
        exp.add_report(
            ComparativeReport(algorithm_pairs, attributes=attributes),
            outfile=outfile)
        exp.add_step(
            'publish-report', subprocess.call, ['publish', outfile])

exp.run_steps()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import itertools
import os

from lab.environments import LocalEnvironment, MaiaEnvironment

from downward.reports.compare import ComparativeReport

import common_setup
from common_setup import IssueConfig, IssueExperiment, RelativeScatterPlotReport


BENCHMARKS_DIR = os.environ["DOWNWARD_BENCHMARKS"]
REVISIONS = ["issue213-base", "issue213-v1", "issue213-v3"]
BUILDS = ["release32", "release64"]
SEARCHES = [
    ("bjolp", "astar(lmcount(lm_merged([lm_rhw(), lm_hm(m=1)]), admissible=true), mpd=true)"),
    ("blind", "astar(blind())"),
    ("cegar", "astar(cegar())"),
    ("divpot", "astar(diverse_potentials())"),
    ("ipdb", "astar(ipdb(max_time=900))"),
    ("lmcut", "astar(lmcut())"),
    ("mas",
        "astar(merge_and_shrink(shrink_strategy=shrink_bisimulation(greedy=false), "
        "merge_strategy=merge_dfp(), "
        "label_reduction=exact(before_shrinking=true, before_merging=false), max_states=100000, threshold_before_merge=1))"),
    ("seq", "astar(operatorcounting([state_equation_constraints()]))"),
]
CONFIGS = [
    IssueConfig(
        "{nick}-{build}".format(**locals()),
        ["--search", search],
        build_options=[build],
        driver_options=["--build", build])
    for nick, search in SEARCHES
    for build in BUILDS
]
SUITE = common_setup.DEFAULT_OPTIMAL_SUITE
ENVIRONMENT = MaiaEnvironment(
    priority=0, email="jendrik.seipp@unibas.ch")

if common_setup.is_test_run():
    SUITE = IssueExperiment.DEFAULT_TEST_SUITE
    ENVIRONMENT = LocalEnvironment(processes=1)

exp = IssueExperiment(
    revisions=REVISIONS,
    configs=CONFIGS,
    environment=ENVIRONMENT,
)
exp.add_suite(BENCHMARKS_DIR, SUITE)
exp.add_absolute_report_step()

attributes = [
    "coverage", "error", "expansions_until_last_jump", "memory",
    "score_memory", "total_time", "score_total_time"]

# Compare revisions.
# lmcut-base-32 vs. lmcut-v1-32 vs. lmcut-v3-32
# lmcut-base-64 vs. lmcut-v1-64 vs. lmcut-v3-64
for build in BUILDS:
    for rev1, rev2 in itertools.combinations(REVISIONS, 2):
        algorithm_pairs = [
            ("{rev1}-{config_nick}-{build}".format(**locals()),
             "{rev2}-{config_nick}-{build}".format(**locals()),
             "Diff ({config_nick}-{build})".format(**locals()))
            for config_nick, search in SEARCHES]
        exp.add_report(
            ComparativeReport(algorithm_pairs, attributes=attributes),
            name="issue213-{rev1}-vs-{rev2}-{build}".format(**locals()))

# Compare builds.
# lmcut-base-32 vs. lmcut-base-64
# lmcut-v1-32 vs. lmcut-v1-64
# lmcut-v3-32 vs. lmcut v3-64
for build1, build2 in itertools.combinations(BUILDS, 2):
    for rev in REVISIONS:
        algorithm_pairs = [
            ("{rev}-{config_nick}-{build1}".format(**locals()),
             "{rev}-{config_nick}-{build2}".format(**locals()),
             "Diff ({config_nick}-{rev})".format(**locals()))
            for config_nick, search in SEARCHES]
        exp.add_report(
            ComparativeReport(algorithm_pairs, attributes=attributes),
            name="issue213-{build1}-vs-{build2}-{rev}".format(**locals()))

# Compare across revisions and builds.
# lmcut-base-32 vs. lmcut-v3-64
build1, build2 = BUILDS
rev1, rev2 = "issue213-base", "issue213-v3"
algorithm_pairs = [
    ("{rev1}-{config_nick}-{build1}".format(**locals()),
     "{rev2}-{config_nick}-{build2}".format(**locals()),
     "Diff ({config_nick})".format(**locals()))
    for config_nick, search in SEARCHES]
exp.add_report(
    ComparativeReport(algorithm_pairs, attributes=attributes),
    name="issue213-before-vs-after")

for attribute in ["total_time", "memory"]:
    exp.add_report(
        RelativeScatterPlotReport(
            attributes=[attribute],
            filter_algorithm=["issue213-v1-blind-release32", "issue213-v3-blind-release32"]),
        name="issue213-relative-scatter-blind-m32-v1-vs-v3-{}".format(attribute))

exp.run_steps()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import itertools
import os

from lab.environments import LocalEnvironment, BaselSlurmEnvironment

from downward.reports.compare import ComparativeReport

import common_setup
from common_setup import IssueConfig, IssueExperiment
from relativescatter import RelativeScatterPlotReport

DIR = os.path.dirname(os.path.abspath(__file__))
BENCHMARKS_DIR = os.environ["DOWNWARD_BENCHMARKS"]
REVISIONS = ["issue213-v7"]
BUILDS = ["release32", "release64"]
SEARCHES = [
    ("bjolp", "astar(lmcount(lm_merged([lm_rhw(), lm_hm(m=1)]), admissible=true), mpd=true)"),
    ("blind", "astar(blind())"),
    ("cegar", "astar(cegar())"),
    ("divpot", "astar(diverse_potentials())"),
    ("ipdb", "astar(ipdb())"),
    ("lmcut", "astar(lmcut())"),
    ("mas",
        "astar(merge_and_shrink(shrink_strategy=shrink_bisimulation(greedy=false),"
        " merge_strategy=merge_sccs(order_of_sccs=topological,"
        " merge_selector=score_based_filtering(scoring_functions=[goal_relevance, dfp, total_order])),"
        " label_reduction=exact(before_shrinking=true, before_merging=false),"
        " max_states=50000, threshold_before_merge=1))"),
    ("seq", "astar(operatorcounting([state_equation_constraints()]))"),
]
CONFIGS = [
    IssueConfig(
        "-".join([search_nick, build]),
        ["--search", search],
        build_options=[build],
        driver_options=["--build", build, "--overall-time-limit", "5m"])
    for rev in REVISIONS
    for build in BUILDS
    for search_nick, search in SEARCHES
]
SUITE = common_setup.DEFAULT_OPTIMAL_SUITE
ENVIRONMENT = BaselSlurmEnvironment(
    partition="infai_2",
    email="jendrik.seipp@unibas.ch",
    export=["PATH", "DOWNWARD_BENCHMARKS"])

if common_setup.is_test_run():
    SUITE = IssueExperiment.DEFAULT_TEST_SUITE
    ENVIRONMENT = LocalEnvironment(processes=1)

exp = IssueExperiment(
    revisions=REVISIONS,
    configs=CONFIGS,
    environment=ENVIRONMENT,
)
exp.add_suite(BENCHMARKS_DIR, SUITE)

exp.add_parser(exp.EXITCODE_PARSER)
exp.add_parser(exp.TRANSLATOR_PARSER)
exp.add_parser(exp.SINGLE_SEARCH_PARSER)
exp.add_parser(exp.PLANNER_PARSER)

exp.add_step('build', exp.build)
exp.add_step('start', exp.start_runs)
exp.add_fetcher(name='fetch')

#exp.add_absolute_report_step()
#exp.add_comparison_table_step()

attributes = IssueExperiment.DEFAULT_TABLE_ATTRIBUTES

# Compare builds.
for build1, build2 in itertools.combinations(BUILDS, 2):
    for rev in REVISIONS:
        algorithm_pairs = [
            ("{rev}-{search_nick}-{build1}".format(**locals()),
             "{rev}-{search_nick}-{build2}".format(**locals()),
             "Diff ({search_nick}-{rev})".format(**locals()))
            for search_nick, search in SEARCHES]
        exp.add_report(
            ComparativeReport(algorithm_pairs, attributes=attributes),
            name="issue213-{build1}-vs-{build2}-{rev}".format(**locals()))

exp.run_steps()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

from downward import suites
from downward.experiment import FastDownwardExperiment
from downward.reports.compare import CompareConfigsReport

from relativescatter import RelativeScatterPlotReport
import common_setup

REPO = common_setup.get_repo_base()
REV_BASE = 'issue585-base'
REV_V1 = 'issue585-v3'
SUITE = suites.suite_optimal_with_ipc11()
ALGORITHMS = {
    'astar_ipdb_base': (REV_BASE, ['--search', 'astar(ipdb())']),
    'astar_ipdb_v3': (REV_V1, ['--search', 'astar(ipdb())']),

    'astar_gapdb_base': (REV_BASE, ['--search', 'astar(gapdb())']),
    'astar_gapdb_v3': (REV_V1, ['--search', 'astar(zopdbs(patterns=genetic()))']),
}
COMPARED_ALGORITHMS = [
    ('astar_ipdb_base', 'astar_ipdb_v3', 'Diff (ipdb)'),
    ('astar_gapdb_base', 'astar_gapdb_v3', 'Diff (gapdb)'),
]

exp = common_setup.IssueExperiment(
    revisions=[],
    configs={},
    suite=SUITE,
)

for nick, (rev, cmd) in ALGORITHMS.items():
    exp.add_algorithm(nick, REPO, rev, cmd)

exp.add_report(CompareConfigsReport(
    COMPARED_ALGORITHMS,
    attributes=common_setup.IssueExperiment.DEFAULT_TABLE_ATTRIBUTES
))

exp.add_report(
    RelativeScatterPlotReport(
        attributes=["total_time"],
        filter_config=["astar_ipdb_base", "astar_ipdb_v3"],
        get_category=lambda run1, run2: run1.get("domain"),
    ),
    outfile='issue585_ipdb_base_v3_total_time.png'
)

exp.add_report(
    RelativeScatterPlotReport(
        attributes=["total_time"],
        filter_config=["astar_gapdb_base", "astar_gapdb_v3"],
        get_category=lambda run1, run2: run1.get("domain"),
    ),
    outfile='issue585_gapdb_base_v3_total_time.png'
)

exp()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

from downward import configs, suites
from downward.reports.scatter import ScatterPlotReport

import common_setup
from relativescatter import RelativeScatterPlotReport


SEARCH_REVS = ["issue529-v1-base", "issue529-v1"]
SUITE = suites.suite_optimal_with_ipc11()

CONFIGS = {
    'astar_blind': [
        '--search',
        'astar(blind())'],
    'astar_ipdb': [
        '--search',
        'astar(ipdb())'],
    'astar_cpdbs': [
        '--search',
        'astar(cpdbs())'],
    'astar_gapdb': [
        '--search',
        'astar(gapdb())'],
    'astar_pdb': [
        '--search',
        'astar(pdb())'],
    'astar_zopdbs': [
        '--search',
        'astar(zopdbs())'],
    'eager_greedy_cg': [
        '--heuristic',
        'h=cg()',
        '--search',
        'eager_greedy(h, preferred=h)'],
}

exp = common_setup.IssueExperiment(
    revisions=SEARCH_REVS,
    configs=CONFIGS,
    suite=SUITE,
    )

exp.add_absolute_report_step()
exp.add_comparison_table_step()

for conf in CONFIGS:
    for attr in ("memory", "total_time"):
        exp.add_report(
            RelativeScatterPlotReport(
                attributes=[attr],
                get_category=lambda run1, run2: run1.get("domain"),
                filter_config=["issue529-v1-base-%s" % conf, "issue529-v1-%s" % conf]
            ),
            outfile='issue529_base_v1_%s_%s.png' % (conf, attr)
        )

exp()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import itertools
import os

from lab.environments import LocalEnvironment, BaselSlurmEnvironment

from downward.reports.compare import ComparativeReport

import common_setup
from common_setup import IssueConfig, IssueExperiment


DIR = os.path.dirname(os.path.abspath(__file__))
BENCHMARKS_DIR = os.environ["DOWNWARD_BENCHMARKS"]
REVISIONS = ["issue731-v5-base", "issue731-v5"]
BUILDS = ["release32", "release64"]
SEARCHES = [
    ("blind", "astar(blind())"),
    ("systematic2", "astar(cpdbs(systematic(2)))"),
    ("ipdb", "astar(ipdb())"),
]
CONFIGS = [
    IssueConfig(
        "{nick}-{build}".format(**locals()),
        ["--search", search],
        build_options=[build],
        driver_options=["--build", build])
    for nick, search in SEARCHES
    for build in BUILDS
]
SUITE = common_setup.DEFAULT_OPTIMAL_SUITE
ENVIRONMENT = BaselSlurmEnvironment(
    email="jendrik.seipp@unibas.ch")

if common_setup.is_test_run():
    SUITE = IssueExperiment.DEFAULT_TEST_SUITE
    ENVIRONMENT = LocalEnvironment(processes=1)

exp = IssueExperiment(
    revisions=REVISIONS,
    configs=CONFIGS,
    environment=ENVIRONMENT,
)
exp.add_suite(BENCHMARKS_DIR, SUITE)
exp.add_comparison_table_step()

# Compare builds.
for build1, build2 in itertools.combinations(BUILDS, 2):
    for rev in REVISIONS:
        algorithm_pairs = [
            ("{rev}-{config_nick}-{build1}".format(**locals()),
             "{rev}-{config_nick}-{build2}".format(**locals()),
             "Diff ({config_nick}-{rev})".format(**locals()))
            for config_nick, search in SEARCHES]
        exp.add_report(
            ComparativeReport(
                algorithm_pairs,
                attributes=IssueExperiment.DEFAULT_TABLE_ATTRIBUTES),
            name="issue731-{build1}-vs-{build2}-{rev}".format(**locals()))

exp.run_steps()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

from downward import suites

import common_setup
import configs


REVS = ["issue508-base", "issue508-v1"]
LIMITS = {"search_time": 1800}
SUITE = suites.suite_optimal_with_ipc11()

configs_optimal_core = configs.default_configs_optimal(ipc=False)
CONFIGS = {}
for nick in ["astar_merge_and_shrink_bisim", "astar_merge_and_shrink_greedy_bisim"]:
    CONFIGS[nick] = configs_optimal_core[nick]

exp = common_setup.IssueExperiment(
    search_revisions=REVS,
    configs=CONFIGS,
    suite=SUITE,
    limits=LIMITS,
    )
exp.add_comparison_table_step()

exp()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import os

from lab.environments import LocalEnvironment, MaiaEnvironment

from common_setup import IssueConfig, IssueExperiment, is_test_run


BENCHMARKS_DIR = os.environ["DOWNWARD_BENCHMARKS"]
REVISIONS = ["issue591-base", "issue591-v1"]
CONFIGS = [
    IssueConfig(
        "lazy_greedy_{}".format(heuristic),
        ["--heuristic", "h={}()".format(heuristic),
         "--search", "lazy_greedy(h, preferred=h)"])
    for heuristic in ["add", "cea", "cg", "ff"]
]
SUITE = [
    'barman-sat14-strips', 'cavediving-14-adl', 'childsnack-sat14-strips',
    'citycar-sat14-adl', 'floortile-sat14-strips', 'ged-sat14-strips',
    'hiking-sat14-strips', 'maintenance-sat14-adl',
    'openstacks-sat14-strips', 'parking-sat14-strips',
    'tetris-sat14-strips', 'thoughtful-sat14-strips',
    'transport-sat14-strips', 'visitall-sat14-strips']
ENVIRONMENT = MaiaEnvironment(
    priority=0, email="jendrik.seipp@unibas.ch")

if is_test_run():
    SUITE = IssueExperiment.DEFAULT_TEST_SUITE
    ENVIRONMENT = LocalEnvironment(processes=1)

exp = IssueExperiment(
    revisions=REVISIONS,
    configs=CONFIGS,
    environment=ENVIRONMENT,
)
exp.add_suite(BENCHMARKS_DIR, SUITE)

exp.add_absolute_report_step()
exp.add_comparison_table_step()
exp.add_scatter_plot_step(attributes=["total_time"])

exp()
